-- Módulo de exportación a OpenSCAD para impresión 3D
-- Por: David Martín Pascual - Marzo 2019
-- Versión: v2.1
-- Licencia: MIT

openscad = {}

openscad.rutamodulo = minetest.get_modpath("openscad")
openscad.rutamundo = minetest.get_worldpath()
openscad.maxlon = 200

dofile(openscad.rutamodulo .. "/tablas.lua")
dofile(openscad.rutamodulo .. "/posicion.lua")

openscad.exportar = function(fichero, x1, y1, z1, x2, y2, z2)

    -- Ruta de salida y verificación del archivo de salida
    local origen, mensajeerror = io.open(openscad.rutamodulo .. "/" .. "modulos.scad", "r")
    if not origen then
		error(mensajeerror)
    end
	local modulos = origen:read("*a")
	origen:close()

	local destino, mensajeerror = io.open(openscad.rutamundo .. "/" .. fichero, "w")
    if not destino then
		error(mensajeerror)
    end

	-- Cabecera y módulos del fichero .scad
    destino:write(string.format('// Volumen exportado de [%d, %d, %d] a [%d, %d, %d]\n\n', x1, y1, z1, x2, y2, z2))
	destino:write(modulos)
    destino:write('union() {\n')

    -- Manipuladores de nodos contenidos entre las coordenadas indicadas
    local vm = minetest.get_voxel_manip()
    local pMin, pMax = vm:read_from_map({ x = x1, y = y1, z = z1 }, { x = x2, y = y2, z = z2 })
    local data = vm:get_data()
    local par2 = vm:get_param2_data()
    local va = VoxelArea:new({ MinEdge = pMin, MaxEdge = pMax })

    -- Comprueba conexión de muros
    local function muro_vecino(x, y, z)
        if va:contains(x, y, z) then
            local ind = va:index(x, y, z)
            local nid = data[ind]
            local nom = minetest.get_name_from_content_id(nid)
            local val = openscad.bloquesadmitidos[nom]
            if (val == 2) or (string.find(nom, 'stone') ~= nil) or (string.find(nom, 'cobble') ~= nil) then
                return true
            end
        end
        return false
    end

    -- Comprueba conexión de vallas
    local function valla_vecina(x, y, z)
        if va:contains(x, y, z) then
            local ind = va:index(x, y, z)
            local nid = data[ind]
            local nom = minetest.get_name_from_content_id(nid)
            local val = openscad.bloquesadmitidos[nom]
            if (val == 4) or (val == 5) then
                return true
            end
        end
        return false
    end

    -- Comprueba conexión de verjas
    local function verja_vecina(x, y, z)
        if va:contains(x, y, z) then
            local ind = va:index(x, y, z)
            local nid = data[ind]
            local nom = minetest.get_name_from_content_id(nid)
            local val = openscad.bloquesadmitidos[nom]
            if (val == 6) then
                return true
            end
        end
        return false
    end

    -- Exploración de bloques a exportar
    for ly = y1, y2 do
		local oz = ly - y1
        for lx = x1, x2 do
            local ox = lx - x1
            local longitud = 0
            local inicio = 0
            for lz = z1, z2 do
                local oy = lz - z1
                local indice = va:index(lx, ly, lz)
                local nodo_id = data[indice]
                local nodo_nom = minetest.get_name_from_content_id(nodo_id)
                local nodo_p2 = par2[indice]
                local valor = openscad.bloquesadmitidos[nodo_nom]
                -- Agrupar bloques para simplificar modelo
                if valor == 1 then
                    if longitud == 0 then
						inicio = oy
					end
                    longitud = longitud + 1
                else
                    if longitud > 0 then
                        destino:write(string.format(' bloque(%d, %d, %d, %d);\n', ox, inicio, oz, longitud))
                        longitud = 0
                    end

					local ve1 = 0;
					local ve2 = 0;
					local ve3 = 0;
					local ve4 = 0;
					local subn = string.sub(nodo_nom, 8, 12);

					if subn == 'slab_' then
						destino:write(string.format(' losa(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2))

					elseif subn == 'stair' then
						local subn2 = string.sub(nodo_nom, 13, 15);
						if subn2 == '_in' then
							destino:write(string.format(' escalon_in(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2))
						elseif subn2 == '_ou' then
							destino:write(string.format(' escalon_out(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2))
						else
							destino:write(string.format(' escalon(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2))
						end

					elseif (string.find(nodo_nom, 'tree') ~= nil) then
						if (string.find(nodo_nom, 'all-faces') ~= nil) then
							destino:write(string.format(' tronco(%d, %d, %d, %d);\n', ox, oy, oz, 0))
							destino:write(string.format(' tronco(%d, %d, %d, %d);\n', ox, oy, oz, 7))
							destino:write(string.format(' tronco(%d, %d, %d, %d);\n', ox, oy, oz, 12))
						else
							destino:write(string.format(' tronco(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2))
						end

					elseif (string.find(nodo_nom, 'leaves') ~= nil) then
						destino:write(string.format(' hojas(%d, %d, %d);\n', ox, oy, oz))

					elseif valor == 2 then
						if muro_vecino(lx, ly, lz+1) then ve1 = 1; end
						if muro_vecino(lx+1, ly, lz) then ve2 = 1; end
						if muro_vecino(lx, ly, lz-1) then ve3 = 1; end
						if muro_vecino(lx-1, ly, lz) then ve4 = 1; end
						destino:write(string.format(' muro(%d, %d, %d, %d, %d, %d, %d);\n', ox, oy, oz, ve1, ve2, ve3, ve4));

					elseif valor == 3 then
						destino:write(string.format(' puerta(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2));

					elseif valor == 4 then
						if valla_vecina(lx, ly, lz+1) then ve1 = 1; end
						if valla_vecina(lx+1, ly, lz) then ve2 = 1; end
						if valla_vecina(lx, ly, lz-1) then ve3 = 1; end
						if valla_vecina(lx-1, ly, lz) then ve4 = 1; end
						destino:write(string.format(' valla(%d, %d, %d, %d, %d, %d, %d);\n', ox, oy, oz, ve1, ve2, ve3, ve4));

					elseif valor == 5 then
						local cerrado = 1
						if (string.find(nodo_nom, '_closed') == nil) then cerrado = 0; end
						destino:write(string.format(' porton(%d, %d, %d, %d, %d);\n', ox, oy, oz, nodo_p2, cerrado));

					elseif valor == 6 then
						if (string.find(nodo_nom, '_flat') == nil) then
							if verja_vecina(lx, ly, lz+1) then ve1 = 1; end
							if verja_vecina(lx+1, ly, lz) then ve2 = 1; end
							if verja_vecina(lx, ly, lz-1) then ve3 = 1; end
							if verja_vecina(lx-1, ly, lz) then ve4 = 1; end
						end
						destino:write(string.format(' verja(%d, %d, %d, %d, %d, %d, %d, %d);\n', ox, oy, oz, ve1, ve2, ve3, ve4, nodo_p2));

					elseif valor == 7 then
						destino:write(string.format(' escalera(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2));

					elseif valor == 8 then
						if (string.find(nodo_nom, 'open') == nil) then nodo_p2 = -1; end
						destino:write(string.format(' trampa(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2));

					elseif valor == 9 then
						destino:write(string.format(' antorcha(%d, %d, %d, %d);\n', ox, oy, oz, nodo_p2));

					elseif openscad.bloquesignorar[nodo_nom] == nil then
						destino:write(string.format(' // Nodo(%d, %d, %d) [%s], p2:%d);\n', ox, oy, oz, nodo_nom, nodo_p2));
					end
                end
            end
            if longitud > 0 then
                destino:write(string.format(' bloque(%d, %d, %d, %d);\n', ox, inicio, oz, longitud))
            end
         end
      end
      destino:write('}\n')
      destino:close()
   end


-- Registro de privilegio específico
minetest.register_privilege("openscad", {
    description = "Permite exportar parte del mapa a fichero",
	give_to_singleplayer = true
	})


-- Registro del comando del chat
minetest.register_chatcommand("openscad", {
    params = "[fichero]",
    description = "Exporta mapa definido por el cubo (pos1) a (pos2) en formato OpenSCAD.",
    privs = { openscad = true },
    func = function(nombre, parametros)
		-- Nombre del fichero
		local lp = string.split(parametros, " ")
		local nf = lp[1]
		if nf == nil then
		   nf = nombre .. ".scad"
		else
		   nf = nombre .. "-" .. nf .. ".scad"
		end

		-- Verificación marcas
		local pos1, pos2 = openscad.marcas(nombre)
		if not (pos1) then
			return false, "No has establecido la Marca 1"
		elseif not (pos2) then
			return false, "No has establecido la Marca 2"
		elseif (pos2.x - pos1.x) > openscad.maxlon then
		   return false, string.format("Error en longitud X (%d -> %d) > %d.", pos1.x, pos2.x, openscad.maxlon)
		elseif (pos2.z - pos1.z) > openscad.maxlon then
		   return false, string.format("Error en longitud Z (%d -> %d) > %d.", pos1.z, pos2.z, openscad.maxlon)
		elseif (pos2.y - pos1.y) > openscad.maxlon then
		   return false, string.format("Error en altura Y (%d -> %d) > %d.", pos1.y, pos2.y, openscad.maxlon)
		else
		   openscad.exportar(nf, pos1.x, pos1.y, pos1.z, pos2.x, pos2.y, pos2.z)
		   return true, "Exportado a " .. nf
		end
	 end
   })