// Módulo de exportación a OpenSCAD para impresión 3D
// Por: David Martín Pascual - Marzo 2019
// Versión: v2.1
// Licencia: MIT

muros     = true;
puertas   = true;
trampas   = true;
vallas    = true;
portones  = true;
escaleras = true;
verjas    = true;
troncos   = true;
hojas     = true;
antorchas = true;

// *********************************************************************

rotaciones = [ [0, 0, 0], [0, 0, 270], [0, 0, 180], [0, 0, 90],
	[270, 0, 0], [270, 270, 0], [270, 180, 0], [270, 90, 0],
	[90, 0, 0], [90, 90, 0], [90, 180, 0], [90, 270, 0],
	[0, 90, 0], [270, 0, 270], [0, 270, 180], [90, 0, 90],
	[0, 270, 0], [90, 0, 270], [0, 90, 180], [270, 0, 90],
	[0, 180, 0], [0, 180, 90], [0, 180, 180], [0, 180, 270] ];

pared = [ [0, 270, 0], [0, 90, 0], [0, 0, 0], [0, 0, 180], [0, 0, 90], [0, 0, 270] ];

module bloque(x, y, z, lon) {
	translate([x, y + (lon-1)/2, z]) cube([1, lon, 1], true);
	}

module escalon(x, y, z, p2) {
    translate([x, y, z])
        rotate(rotaciones[p2])
            union() {
				translate([0, 0, -0.25])   cube([1, 1, 0.5], true);
				translate([0, 0.25, 0.25]) cube([1, 0.5, 0.5], true);
				}
    }

module escalon_out(x, y, z, p2) {
    translate([x, y, z])
        rotate(rotaciones[p2])
            union() {
				translate([0, 0, -0.25]) cube([1, 1, 0.5], true);
				translate([-0.25, 0.25, 0.25]) cube([0.5, 0.5, 0.5], true);
				}
    }


module escalon_in(x, y, z, p2) {
    translate([x, y, z])
        rotate(rotaciones[p2])
            difference() {
				cube([1, 1, 1], true);
				translate([0.25, -0.25, 0.25]) cube([0.5, 0.5, 0.5], true);
				}
    }

module losa(x, y, z, p2) {
    translate([x, y, z])
        rotate(rotaciones[p2])
            translate([0, 0, -0.25]) cube([1, 1, 0.5], true);
    }


module tronco(x, y, z, p2) {
	if (troncos) translate([x, y, z]) rotate(rotaciones[p2])
		rotate([0, 0, 22.5]) cylinder(1, 0.5, 0.5, $fn=8, true);
	}

module hojas(x, y, z) {
    $fn = 10;
	if (hojas) translate([x, y, z]) sphere(d=1.5);
	}

module muro(x, y, z, v1, v2, v3, v4) {
	if (muros) translate([x, y, z]) {
        cube([0.33, 0.33, 1], true);
        if (v1 == 1) translate([0, 0.33, -0.13]) cube([0.25, 0.35, 0.75], true);
        if (v2 == 1) translate([0.33, 0, -0.13]) cube([0.35, 0.25, 0.75], true);
        if (v3 == 1) translate([0, -0.33, -0.13]) cube([0.25, 0.35, 0.75], true);
        if (v4 == 1) translate([-0.33, 0, -0.13]) cube([0.35, 0.25, 0.75], true);
        }
	}

module valla(x, y, z, v1, v2, v3, v4) {
	if (vallas) translate([x, y, z]) {
		cube([0.25, 0.25, 1.002], true);
		if (v1 == 1) {
            translate([0, 0.3, 0.2]) cube([0.2, 0.42, 0.2], true);
            translate([0, 0.3, -0.2]) cube([0.2, 0.42, 0.2], true);
            }
        if (v2 == 1) {
            translate([0.3, 0, 0.2]) cube([0.42, 0.2, 0.2], true);
            translate([0.3, 0, -0.2]) cube([0.42, 0.2, 0.2], true);
            }
        if (v3 == 1) {
            translate([0, -0.3, 0.2]) cube([0.2, 0.42, 0.2], true);
            translate([0, -0.3, -0.2]) cube([0.2, 0.42, 0.2], true);
            }
        if (v4 == 1) {
            translate([-0.3, 0, 0.2]) cube([0.42, 0.2, 0.2], true);
            translate([-0.3, 0, -0.2]) cube([0.42, 0.2, 0.2], true);
            }
		}
	}

module porton(x, y, z, p2, a) {
	if (portones) translate([x, y, z]) rotate(rotaciones[p2]) {
		translate([-0.5, 0, 0]) cube([0.25, 0.25, 1.002], true);
		translate([0.5, 0, 0]) cube([0.25, 0.25, 1.002], true);
        if (a == 0) {
            translate([-0.5, -0.5, 0.2]) rotate([0, 0, -90]) cube([1, 0.2, 0.2], true);
            translate([-0.5, -0.5, 0]) rotate([0, -25 ,-90]) cube([1, 0.2, 0.2], true);
            }
        else {
            translate([0, 0, 0.2]) cube([1, 0.2, 0.2], true);
            rotate([0, -25 ,0]) cube([1, 0.2, 0.2], true);
            }
        }
	}

module escalera(x, y, z, p2) {
	$fn=8;
	if (escaleras && p2 < 6) translate([x, y, z]) rotate(pared[p2]) union() {
		translate([0.4, -0.4, -0.5]) cylinder(r=0.1, h=1);
		translate([0.4, 0.4, -0.5])  cylinder(r=0.1, h=1);
        rotate([90, 0, 0]) {
            translate([0.4, 0.38, -0.5]) cylinder(r=0.05, h=1);
            translate([0.4, 0.13, -0.5]) cylinder(r=0.05, h=1);
            translate([0.4, -0.13, -0.5]) cylinder(r=0.05, h=1);
            translate([0.4, -0.38, -0.5]) cylinder(r=0.05, h=1);
            }
		}
	}

module puerta(x, y, z, p2) {
    rotar = [0, -90, 180, 90];
	if (puertas) translate([x, y, z]) rotate([0, 0, rotar[p2]])
       translate([-0.5, -0.4, -0.50]) difference() {
            cube([1.001, 0.1, 2.001]);
            translate([0.1, -0.1, 0.1]) cube([0.8, 0.12, 0.8]);
            translate([0.1, 0.08, 0.1]) cube([0.8, 0.1, 0.8]);
            translate([0.1, -0.1, 1.1]) cube([0.8, 0.12, 0.8]);
            translate([0.1, 0.08, 1.1]) cube([0.8, 0.1, 0.8]);
            translate([0.2, -0.1, 1.2]) cube([0.2, 0.2, 0.3]);
            translate([0.2, -0.1, 1.6]) cube([0.2, 0.2, 0.3]);
            translate([0.6, -0.1, 1.2]) cube([0.2, 0.2, 0.3]);
            translate([0.6, -0.1, 1.6]) cube([0.2, 0.2, 0.3]);
            }
	}

module antorcha(x, y, z, p2) {
    if (antorchas) translate([x, y, z]) {
        $fn=5;
        sphere(0.19);
		cylinder(r1=0.13, r2=0, h=0.4);
        if (p2 == 1) rotate([0, 180, 0]) cylinder(r=0.08, h=0.6);
        if (p2 == 2) rotate([0, 125, 0]) cylinder(r=0.08, h=0.7);
        if (p2 == 3) rotate([0, 125, 180]) cylinder(r=0.08, h=0.7);
        if (p2 == 4) rotate([0, 125, 90]) cylinder(r=0.08, h=0.7);
        if (p2 == 5) rotate([0, 125, 270]) cylinder(r=0.08, h=0.7);
        }
	}

module trampa(x ,y, z, p2) {
    if (trampas) translate([x, y, z]) rotate(p2 == -1 ? [-90,0,0] : rotaciones[p2]) {
		difference() {
            translate([0, 0.4, 0]) cube([1.001, 0.2, 1.001], true);
            translate([0, 0.3, 0]) cube([0.8, 0.12, 0.8], true);
            translate([0, 0.5, 0]) cube([0.8, 0.12, 0.8], true);
            translate([0.2, 0.4, 0.2]) cube([0.25, 0.5, 0.25], true);
            translate([-0.2, 0.4, -0.2]) cube([0.25, 0.5, 0.25], true);
            translate([0.2, 0.4, -0.2]) cube([0.25, 0.5, 0.25], true);
            translate([-0.2, 0.4, 0.2]) cube([0.25, 0.5, 0.25], true);
            }
		}
	}

module mitad_verja(r) {
	rotate([0, 0, r]) translate([0.25, 0, 0]) difference() {
		cube([0.501, 0.2, 1.002], true);
		cube([0.3, 0.4, 0.801],true);
		}
	}

module verja(x, y, z, v1, v2, v3, v4, p2) {
	 if (verjas) translate([x, y, z]) {
        if ((v1 + v2 + v3 + v4) == 0) {
            if (p2 == 0) { mitad_verja(0); mitad_verja(180);}
            if (p2 == 3) { mitad_verja(90); mitad_verja(270);}
            }
		if (v1 == 1) mitad_verja(90);
		if (v2 == 1) mitad_verja(0);
		if (v3 == 1) mitad_verja(-90);
		if (v4 == 1) mitad_verja(180);
		}
	}

// Datos del modelo