-- Módulo de exportación a OpenSCAD para impresión 3D
-- Por: David Martín Pascual - Marzo 2019
-- Versión v2.1

-- Código de marcas de posición original de los módulos "WorldEdit" y "Areas"
-- Replicado para evitar dependencias y soportar marcas de múltiples jugadores


openscad.marcas1 = {}
openscad.marcas2 = {}
openscad.pos1 = {}
openscad.pos2 = {}


-- Marcar el primer punto
minetest.register_chatcommand("openscad_p1", {
	params = "[X Y Z|X,Y,Z]",
	description = "Marca la primera posición del volumen a exportar en tu posicion actual o la especificada",
	privs = {},
	func = function(nombre, parametros)
		local posicion = nil
		local partes, _, x, y, z = parametros:find("^(-?%d+)[, ](-?%d+)[, ](-?%d+)$")
		if partes then
			posicion = {x=tonumber(x), y=tonumber(y), z=tonumber(z)}
		elseif parametros == "" then
			local jugador = minetest.get_player_by_name(nombre)
			if jugador then
				posicion = jugador:getpos()
			else
				return false, "Imposible obtener la posición."
			end
		else
			return false, "Uso incorrecto, consulta /help openscad_p1"
		end
		posicion = vector.round(posicion)
		openscad.pos1[nombre] = posicion
		openscad.marcaPos1(nombre)
		return true, "Marca 1 en " .. minetest.pos_to_string(posicion)
	end,
})


-- Marcar el segundo punto
minetest.register_chatcommand("openscad_p2", {
	params = "[X Y Z|X,Y,Z]",
	description = "Marca la segunda posición del volumen a exportar en tu posicion actual o la especificada",
	func = function(nombre, parametros)
		local posicion = nil
		local partes, _, x, y, z = parametros:find("^(-?%d+)[, ](-?%d+)[, ](-?%d+)$")
		if partes then
			posicion = {x=tonumber(x), y=tonumber(y), z=tonumber(z)}
		elseif parametros == "" then
			local jugador = minetest.get_player_by_name(nombre)
			if jugador then
				posicion = jugador:getpos()
			else
				return false, "Imposible obtener la posición."
			end
		else
			return false, "Uso incorrecto, consulta /help openscad_p2"
		end
		posicion = vector.round(posicion)
		openscad.pos2[nombre] = posicion
		openscad.marcaPos2(nombre)
		return true, "Marca 2 en " .. minetest.pos_to_string(posicion)
	end,
})


-- Obtener las marcas con los valores ordenados
openscad.marcas = function(nombre)
	local pos1, pos2 = openscad.pos1[nombre], openscad.pos2[nombre]
	if not (pos1 and pos2) then
		return nil
	end
	pos1, pos2 = vector.new(pos1), vector.new(pos2)
	if pos1.x > pos2.x then
		pos2.x, pos1.x = pos1.x, pos2.x
	end
	if pos1.y > pos2.y then
		pos2.y, pos1.y = pos1.y, pos2.y
	end
	if pos1.z > pos2.z then
		pos2.z, pos1.z = pos1.z, pos2.z
	end
	return pos1, pos2
end


-- Marca en el mundo de la primera posición
openscad.marcaPos1 = function(nombre)
	local posicion = openscad.pos1[nombre]
	if openscad.marcas1[nombre] ~= nil then -- Ya existe marca
		openscad.marcas1[nombre]:remove() -- Elimina la marca
		openscad.marcas1[nombre] = nil
	end
	if posicion ~= nil then -- Pone la marca
		openscad.marcas1[nombre] = minetest.add_entity(posicion, "openscad:pos1")
		openscad.marcas1[nombre]:get_luaentity().active = true
	end
end


-- Marca en el mundo de la segunda posición
openscad.marcaPos2 = function(nombre)
	local posicion = openscad.pos2[nombre]
	if openscad.marcas2[nombre] ~= nil then -- Ya existe marca
		openscad.marcas2[nombre]:remove() -- Elimina la marca
		openscad.marcas2[nombre] = nil
	end
	if posicion ~= nil then -- Pone la marca
		openscad.marcas2[nombre] = minetest.add_entity(posicion, "openscad:pos2")
		openscad.marcas2[nombre]:get_luaentity().active = true
	end
end


-- Registro del bloque para la primera marca
minetest.register_entity("openscad:pos1", {
	initial_properties = {
		visual = "cube",
		visual_size = {x=1.1, y=1.1},
		textures = {"openscad_pos1.png", "openscad_pos1.png", "openscad_pos1.png",
		            "openscad_pos1.png", "openscad_pos1.png", "openscad_pos1.png"},
		collisionbox = {-0.55, -0.55, -0.55, 0.55, 0.55, 0.55},
	},
	on_step = function(self, dtime)
		if self.active == nil then
			self.object:remove()
		end
	end,
	on_punch = function(self, hitter)
		self.object:remove()
		local nombre = hitter:get_player_name()
		openscad.marcas1[nombre] = nil
	end,
})


-- Registro del bloque para la segunda marca
minetest.register_entity("openscad:pos2", {
	initial_properties = {
		visual = "cube",
		visual_size = {x=1.1, y=1.1},
		textures = {"openscad_pos2.png", "openscad_pos2.png", "openscad_pos2.png",
		            "openscad_pos2.png", "openscad_pos2.png", "openscad_pos2.png"},
		collisionbox = {-0.55, -0.55, -0.55, 0.55, 0.55, 0.55},
	},
	on_step = function(self, dtime)
		if self.active == nil then
			self.object:remove()
		end
	end,
	on_punch = function(self, hitter)
		self.object:remove()
		local nombre = hitter:get_player_name()
		openscad.marcas2[nombre] = nil
	end,
})
